@@ -2,8 +2,10 @@
   "Main encoding and decoding namespace."
   (:require [cheshire.factory :as factory]
             [cheshire.generate :as gen]
+            [cheshire.generate-seq :as gen-seq]
             [cheshire.parse :as parse])
   (:import (com.fasterxml.jackson.core JsonParser JsonFactory
+                                       JsonGenerator
                                        JsonGenerator$Feature)
            (com.fasterxml.jackson.dataformat.smile SmileFactory)
            (java.io StringWriter StringReader BufferedReader BufferedWriter
@@ -56,6 +58,43 @@
        (.flush generator)
        writer)))
 
+(defn create-generator [writer]
+  "Returns JsonGenerator for given writer."
+  (.createJsonGenerator
+   ^JsonFactory (or factory/*json-factory*
+                    factory/json-factory) writer))
+
+(def ^:dynamic ^JsonGenerator *generator*)
+(def ^:dynamic *opt-map*)
+
+(defmacro with-writer [[writer opt-map] & body]
+  "Start writing for series objects using the same json generator.
+   Takes writer and options map as arguments.
+   Expects it's body as sequence of write calls.
+   Returns a given writer."
+  `(let [c-wr# ~writer]
+     (binding [*generator* (create-generator c-wr#)
+               *opt-map* ~opt-map]
+       ~@body
+       (.flush *generator*)
+       c-wr#)))
+
+(defn write
+  "Write given Clojure object as a piece of data within with-writer.
+  List of wholeness acceptable values:
+  - no value - the same as :all
+  - :all - write object in a regular way with start and end borders
+  - :start - write object with start border only
+  - :start-inner - write object and it's inner object with start border only
+  - :end - write object with end border only."
+  ([obj] (write obj nil))
+  ([obj wholeness]
+     (gen-seq/generate *generator* obj (or (:date-format *opt-map*)
+                                           factory/default-date-format)
+                       (:ex *opt-map*)
+                       (:key-fn *opt-map*)
+                       :wholeness wholeness)))
+
 (defn generate-smile
   "Returns a SMILE-encoded byte-array for the given Clojure object.
   Takes an optional date format string that Date objects will be encoded with.
@@ -76,6 +115,26 @@
        (.flush generator)
        (.toByteArray baos))))
 
+(defn generate-cbor
+  "Returns a CBOR-encoded byte-array for the given Clojure object.
+  Takes an optional date format string that Date objects will be encoded with.
+
+  The default date format (in UTC) is: yyyy-MM-dd'T'HH:mm:ss'Z'"
+  ([obj]
+     (generate-cbor obj nil))
+  ([obj opt-map]
+     (let [baos (ByteArrayOutputStream.)
+           generator (.createJsonGenerator ^CBORFactory
+                                           (or factory/*cbor-factory*
+                                               factory/cbor-factory)
+                                           baos)]
+       (gen/generate generator obj (or (:date-format opt-map)
+                                       factory/default-date-format)
+                     (:ex opt-map)
+                     (:key-fn opt-map))
+       (.flush generator)
+       (.toByteArray baos))))
+
 ;; Parsers
 (defn parse-string
   "Returns the Clojure object corresponding to the given JSON-encoded string.
@@ -94,6 +153,26 @@
                            (StringReader. string))
         key-fn nil array-coerce-fn))))
 
+;; Parsing strictly
+(defn parse-string-strict
+  "Returns the Clojure object corresponding to the given JSON-encoded string.
+  An optional key-fn argument can be either true (to coerce keys to keywords),
+  false to leave them as strings, or a function to provide custom coercion.
+
+  The array-coerce-fn is an optional function taking the name of an array field,
+  and returning the collection to be used for array values.
+
+  Does not lazily parse top-level arrays."
+  ([string] (parse-string-strict string nil nil))
+  ([string key-fn] (parse-string-strict string key-fn nil))
+  ([^String string key-fn array-coerce-fn]
+     (when string
+       (parse/parse-strict
+        (.createJsonParser ^JsonFactory (or factory/*json-factory*
+                                            factory/json-factory)
+                           (StringReader. string))
+        key-fn nil array-coerce-fn))))
+
 (defn parse-stream
   "Returns the Clojure object corresponding to the given reader, reader must
   implement BufferedReader. An optional key-fn argument can be either true (to
@@ -128,6 +207,22 @@
                                              factory/smile-factory) bytes)
         key-fn nil array-coerce-fn))))
 
+(defn parse-cbor
+  "Returns the Clojure object corresponding to the given CBOR-encoded bytes.
+  An optional key-fn argument can be either true (to coerce keys to keywords),
+  false to leave them as strings, or a function to provide custom coercion.
+
+  The array-coerce-fn is an optional function taking the name of an array field,
+  and returning the collection to be used for array values."
+  ([bytes] (parse-cbor bytes nil nil))
+  ([bytes key-fn] (parse-cbor bytes key-fn nil))
+  ([^bytes bytes key-fn array-coerce-fn]
+     (when bytes
+       (parse/parse
+        (.createJsonParser ^CBORFactory (or factory/*cbor-factory*
+                                            factory/cbor-factory) bytes)
+        key-fn nil array-coerce-fn))))
+
 (def ^{:doc "Object used to determine end of lazy parsing attempt."}
   eof (Object.))
 
@@ -136,7 +231,7 @@
   "Internal lazy-seq parser"
   [^JsonParser parser key-fn array-coerce-fn]
   (lazy-seq
-   (let [elem (parse/parse parser key-fn eof array-coerce-fn)]
+   (let [elem (parse/parse-strict parser key-fn eof array-coerce-fn)]
      (when-not (identical? elem eof)
        (cons elem (parsed-seq* parser key-fn array-coerce-fn))))))
 
@@ -176,5 +271,6 @@
 (def encode-stream generate-stream)
 (def encode-smile generate-smile)
 (def decode parse-string)
+(def decode-strict parse-string-strict)
 (def decode-stream parse-stream)
 (def decode-smile parse-smile)
