@@ -246,7 +246,7 @@
   [db-conn-pool product-name]
   (sql/with-connection db-conn-pool
     (scf-store/validate-database-version #(System/exit 1))
-    (migrate!)
+    (migrate! db-conn-pool)
     (indexes! product-name)))
 
 (defn init-with-db
@@ -310,8 +310,7 @@
                       "might be due to KahaDB corruption. Consult the "
                       "PuppetDB troubleshooting guide.")
                      (throw e)))
-          mq-factory (mq/activemq-connection-factory
-                      (add-max-framesize max-frame-size mq-addr))
+          mq-factory (mq/activemq-connection-factory mq-connection-str)
           mq-connection (doto (.createConnection mq-factory)
                           (.setExceptionListener
                            (reify ExceptionListener
