@@ -140,13 +140,18 @@
   function basically just checks that the operator is known, and then
   dispatches to the function implementing it."
   [ops [op & args :as term]]
-  (when-not (sequential? term)
-    (throw (IllegalArgumentException. (format "%s is not well-formed: queries must be an array" (vec term)))))
-  (when-not op
-    (throw (IllegalArgumentException. (format "%s is not well-formed: queries must contain at least one operator" (vec term)))))
-  (if-let [f (ops op)]
-    (apply f args)
-    (throw (IllegalArgumentException. (format "%s is not well-formed: query operator '%s' is unknown" (vec term) op)))))
+
+  (cond
+    (empty? term)
+    {:where nil :params nil}
+
+    (not op)
+    (throw (IllegalArgumentException. (format "%s is not well-formed: queries must contain at least one operator" (vec term))))
+
+    :else
+    (if-let [f (ops op)]
+      (apply f args)
+      (throw (IllegalArgumentException. (format "%s is not well-formed: query operator '%s' is unknown" (vec term) op))))))
 
 (defn compile-boolean-operator*
   "Compile a term for the boolean operator `op` (AND or OR) applied to
@@ -240,6 +245,27 @@
    "containing_class"       ["resource_events"]
    "name"                   ["environments" "environment"]})
 
+(def resource-event-columns
+  {"certname"               ["latest_events"]
+   "configuration_version"  ["latest_events"]
+   "run_start_time"         ["latest_events"]
+   "run_end_time"           ["latest_events"]
+   "report_receive_time"    ["latest_events"]
+   "hash"                   ["latest_events" "report"]
+   "status"                 ["latest_events"]
+   "timestamp"              ["latest_events"]
+   "resource_type"          ["latest_events"]
+   "resource_title"         ["latest_events"]
+   "property"               ["latest_events"]
+   "new_value"              ["latest_events"]
+   "old_value"              ["latest_events"]
+   "message"                ["latest_events"]
+   "file"                   ["latest_events"]
+   "line"                   ["latest_events"]
+   "containment_path"       ["latest_events"]
+   "containing_class"       ["latest_events"]
+   "name"                   ["latest_events" "environment"]})
+
 (def report-columns
   "Return the queryable set of fields and corresponding table names where they reside"
   {"hash"                  "reports"
@@ -333,7 +359,7 @@
         columns (case kind
                   :fact fact-columns
                   :resource resource-columns
-                  :event event-columns
+                  :event resource-event-columns
                   :report report-columns
                   :factset factset-columns)
         qualified-field (qualified-column field columns)]
@@ -554,9 +580,9 @@
                                                                                                                  args))))))
 
   (let [timestamp-fields {"timestamp"           "resource_events.timestamp"
-                          "run_start_time"      "reports.start_time"
-                          "run_end_time"        "reports.end_time"
-                          "report_receive_time" "reports.receive_time"}]
+                          "run_start_time"      "latest_events.run_start_time"
+                          "run_end_time"        "latest_events.run_end_time"
+                          "report_receive_time" "latest_events.report_receive_time"}]
     (match [path]
            [(field :guard (kitchensink/keyset timestamp-fields))]
            (if-let [timestamp (to-timestamp value)]
@@ -579,23 +605,23 @@
     (let [path (utils/dashes->underscores path)]
       (match [path]
              ["certname"]
-             {:where "reports.certname = ?"
+             {:where "latest_events.certname = ?"
               :params [value]}
 
              ["report"]
-             {:where (format "%s = ?" (sutils/sql-hash-as-str "reports.hash"))
+             {:where (format "%s = ?" (sutils/sql-hash-as-str "latest_events.hash"))
               :params [value]}
 
              ["latest_report?"]
-             {:where (format "resource_events.report_id %s (SELECT certnames.latest_report FROM certnames)"
+             {:where (format "latest_report.report_id %s (SELECT certnames.latest_report FROM certnames)"
                              (if value "IN" "NOT IN"))}
 
              ["environment"]
-             {:where "environments.name = ?"
+             {:where "latest_events.name = ?"
               :params [value]}
 
              [(field :guard #{"report" "resource_type" "resource_title" "status"})]
-             {:where (format "resource_events.%s = ?" field)
+             {:where (format "latest_events.%s = ?" field)
               :params [value] }
 
              ;; these fields allow NULL, which causes a change in semantics when
@@ -603,9 +629,9 @@
              ;; about the NULL case.
              [(field :guard #{"property" "message" "file" "line" "containing_class"})]
              (if-not (nil? value)
-               {:where (format "resource_events.%s = ? AND resource_events.%s IS NOT NULL" field field)
+               {:where (format "latest_events.%s = ? AND latest_events.%s IS NOT NULL" field field)
                 :params [value] }
-               {:where (format "resource_events.%s IS NULL" field)
+               {:where (format "latest_events.%s IS NULL" field)
                 :params nil })
 
              ;; these fields require special treatment for NULL (as described above),
@@ -630,11 +656,11 @@
     (let [path (utils/dashes->underscores path)]
       (match [path]
              ["certname"]
-             {:where (legacy-sql-regexp-match "reports.certname")
+             {:where (legacy-sql-regexp-match "latest_events.certname")
               :params [pattern]}
 
              ["environment"]
-             {:where (legacy-sql-regexp-match "environments.name")
+             {:where (legacy-sql-regexp-match "latest_events.name")
               :params [pattern]}
 
              [(field :guard #{"report" "resource_type" "resource_title" "status"})]
@@ -645,8 +671,8 @@
              ;; wrapped in a NOT(...) clause, so we have to be very explicit
              ;; about the NULL case.
              [(field :guard #{"property" "message" "file" "line" "containing_class"})]
-             {:where (format "%s AND resource_events.%s IS NOT NULL"
-                             (legacy-sql-regexp-match (format "resource_events.%s" field))
+             {:where (format "%s AND latest_events.%s IS NOT NULL"
+                             (legacy-sql-regexp-match (format "latest_events.%s" field))
                              field)
               :params [pattern]}
 
